;Example POOL program, you may use it,
;modify and distribute under GPL license.
;Authors: R&D Independent Team
;Source: www.pool-rnd.com

ht pu                        ;initially hidden, not drawing
:pool_cfg,"refresh := false  ;refresh timer will be created

;base class:
to linked_node
  let "clicked false
  let "fixed false

  ;list of connected nodes, fill it on your own!
  let "links []

  let "gravity {0 0}
  let "loss 0.01 ;loss coefficient (also helps stability)
  let "ampl 0.01 ;force coefficient
  let "mass 0.05 ;node mass
  let "v 0       ;node speed
  ht             ;initially hidden

  ;final setup, should be called from derived class
  ;when links are ready
  to initialized :r :px :py
    "pstart := newarray count :links
    :gravity,1 *= :mass
    :gravity,2 *= :mass
    setxy :px :py ;set initial position
    setr :r       ;radius used as a distance of "zero force"
    st            ;show the turtle when setup is completed
  end

  ;simple, attracting-repelling prototype
  to fcoeff :d
    op :d - radius
  end

  to sum_forces :fxy [:dist 0] [:coeff 0]
    :fxy,1 := :gravity,1
    :fxy,2 := :gravity,2
    let "pi pos @ :links      ;positions of nodes
    foreach "i :pi [
      "dist := distance :i
      "coeff := :ampl * (fcoeff :dist) / :dist
      :fxy,1 += :coeff * (:i,1 - xcor)
      :fxy,2 += :coeff * (:i,2 - ycor)
    ]
  end

  let "force {0 0}
  to onsignalforces
    if !(:fixed || :clicked) [sum_forces :force]
  end

  to on_step_action end ;empty prototype

  to do_step :dt :vd [:vn 0] [:ds 0]
    foreach "i :links [:pstart,repcount := distance :i]

    "dt *= 0.01 ;time interval
    if :dt > 1 ["dt := 1]

    :vd,1 := :vd,1 * :v + :force,1 * :dt / :mass
    :vd,2 := :vd,2 * :v + :force,2 * :dt / :mass
    "vn := sqrt :vd,1^2 + :vd,2^2
    if :vn > 50 ["vn := 50]
    setheaddir :vd
    fd :vn * :dt

    op :vn
  end

  local "pstart  ;distances to nodes before each step
  to onsignalstep :turtle :data
    if !(:fixed || :clicked) [
      "v := do_step :data movedir
      on_step_action
    ]
  end

  to onsignalloss
    if !(:fixed || :clicked) [
      foreach "i :links [
        "v -= :loss * abs :pstart,repcount - distance :i
      ]
      if :v < 0 ["v := 0]
    ]
  end

  to onmousemove :mousepos
    if :clicked && not :fixed [setpos :mousepos]
  end

  to onturtleclicked
    tto this "clicked := true
  end

  to onmousedown
    eventenable "onmousemove
  end

  to onmouseup
    eventdisable "onmousemove
    "clicked := false
  end
end

