;Example POOL program, you may use it,
;modify and distribute under GPL license.
;Authors: R&D Independent Team
;Source: www.pool-rnd.com

include "linked_node.l

to node
  use linked_node

  to fcoeff :d ;overriden default coefficient formula
    if :d > radius [op :d - radius]
    op 0 ;no force at all if distance below radius
  end

  to initialize :n :px :py :h
    "ampl := 0.5
    "loss := 0.03
    "mass := 0.5
    "gravity := {0 -0.6}
    seth :h

    if :n = 1 [
      eventdisable "onsignalforces
      eventdisable "onsignalstep
      eventdisable "onturtleclicked
      eventdisable "onmouseup
      eventdisable "onmousemove
      "fixed := true
    ]

    if :n > 1 [push :links :a,(:n - 1)]
    if :n < :nnodes [push :links :a,(:n + 1)]

    ;call always at the end of initialization:
    initialized 10 :px :py
  end

  signal "ready
end

bounce
setpc {50 0 0}

shared [a nnodes go]
"go := false
"nnodes := 25

; --- 1. --- create nodes
"a := newarray :nnodes ;array of nodes
for [i 1 :nnodes] [:a,:i := (newt $node)]
(waitsignal "ready children)

; --- 2. --- initialize nodes
setxy 0 180 seth 180
for [i 1 :nnodes] [
  (initialize :i xcor ycor heading) @ :a,:i
  rt rnorm 15
  fd 15
]

"switch := button "Start [20 10 70 24]
setonclick :switch [
  "go := not :go
  ifelse :go
  [
    settext :switch "Stop
    "clock := timer [
      signalw "forces
      (signalw "step :dt)
      signal "loss
    ] 8
  ] [
    settext :switch "Start
    stoptimer :clock
  ]
]

to draw_string
  clean
  for [i 2 :nnodes] [(line :a,(:i - 1) :a,:i)]
  refresh
end

"painter := timer [if :go [draw_string]] 40

draw_string

